﻿using System;
using System.Data.Common;
using EPiServer.Common.Data;

namespace ExampleProjects.Movie.Data
{
    public class MovieEntityProvider : IEntityProvider
    {
        private static readonly MovieEntityProvider _instance = 
            new MovieEntityProvider();

        public static IEntityProvider GetProviderInstance()
        {
            return _instance;
}

        public object GetEntityInstance(Type type, int id)
        {
            ValidateIsSupportedType(type);

            return MovieHandler.GetMovie(id);
        }

        public object GetEntityInstance(
            Type type, DbDataReader reader)
        {
            ValidateIsSupportedType(type);

            return ConstructMovie(reader);
        }

        private void ValidateIsSupportedType(Type type)
        {
            if (type == typeof(Movie))
                return;

            throw new NotSupportedException(
                string.Format(
                    "Unsupported type. Cannot create instance of {0}.",
                    type.Name));
        }

        public Movie ConstructMovie(DbDataReader reader)
        {
            int id = reader.GetInt32(0);
            string title = reader.GetString(1);
            int? runtime = null;
            if(!reader.IsDBNull(2))
                runtime = reader.GetInt32(2);
            int authorID = reader.GetInt32(3);

            return new Movie(id, title, runtime, authorID);
        }
    }
}
